#! /usr/bin/env racket
#lang racket

(provide maiden2latlong)

;; Convert a Maidenhead locator to latitude and longitude
;;
;; The locator is expected as a list of dotted pairs.
;;    e.g. IO85jw is expected as ((#\I . #\O)(8 . 5)(#\J . #\W)) 
;; note that letters are uppercase only
;; Character pairs encode longitude first, and then latitude.
;; The first pair (a field) encodes with base 18 and the letters "A" to "R".
;; The second pair (square) encodes with base 10 and the digits "0" to "9".
;; The third pair (subsquare) encodes with base 24 and the letters "A" to "X".
;; The fourth pair (extended square) encodes with base 10 and the digits "0" to "9".
;;
;; The latitude and longitude is returned as a pair of degrees e.g. (55.91667 . -3.16667)
;;
;; TODO: return the centre of the square rather than a corner

(define (maiden2latlong locator)
  (cons (- (/ (maiden2lat locator) 3600) 90)
        (- (/ (maiden2long locator) 3600) 180)))

;; the base to treat each pair as
(define pair-bases '(18 10 24 10))

;; the size of latitude and longitude in seconds
;; for field, square and subsquare
;; TODO extended square
(define lat-dividers '(36000 3600 150))
(define long-dividers '(72000 7200 300))
  
(define (maiden2lat locator)
  (maiden2lat1 locator pair-bases lat-dividers 0))

(define (maiden2lat1 locator bases dividers accumulator)
  (if (null? locator)
      accumulator
      (+ accumulator
         (maiden2lat1 (cdr locator) (cdr bases) (cdr dividers) 
                       (* (decode-seconds (cdar locator) (car bases)) (car dividers))))))

(define (decode-seconds value base)
	(if (> base 10)
	  (- (char->integer value) (char->integer #\A))
	  value))

(define (maiden2long locator)
  (maiden2long1 locator pair-bases long-dividers 0))

(define (maiden2long1 locator bases dividers accumulator)
  (if (null? locator)
      accumulator
      (+ accumulator
         (maiden2long1 (cdr locator) (cdr bases) (cdr dividers) 
                       (* (decode-seconds (caar locator) (car bases)) (car dividers))))))

;; testing
;;(define test-loc (list (cons #\I #\O) (cons 8 5) (cons #\J #\W)))
;;(maiden2latlong test-loc)

    
  
